      program disco
c23456789012345678901234567890123456789012345678901234567890123456789012
C----
C---- generates protein structures from a set of distance bounds
C----
C---- an iterative procedure is used that stops when all distance
C---- bounds are fulfilled. starting coordinates are chosen randomly.
C----
C---- the following input is required:
C----
C---- -reference pdb coordinate file, from which distances are defined
C---- -reference distances + margins, generated by 'dist'
C----
C---- by Bert de Groot, 1996-1998
C----
C----
      implicit none

      include 'params.f'

      integer i,j,k,nat,nfrm,nrit,nstruc,n1,n2,count,
     &     iimp(4,maxat),nimp,id1,ret,resnr(maxat),nviol,nres,
     &     ndis,in(2,maxdis),iseed,nargs,iargc,nit,
     &     gvflen,nfit,chiropt,tag(maxat),idum,nsmoo,maxviol

      integer*8 ntot

      character*5 atnam(maxat),resnam(maxat),dum1

      character*4 cdum1,cdum2

      character*1 chainid(maxat)

      logical fitsel(maxat),had2,xtcflg,pdbflg,restflg,havextc,
     &     traflg,gvfisf,verbflg,fitflg,notall

      real*4 x(3*maxat),davran,meansig(2,maxdis),
     &     lbub(2,maxdis),mass(maxat),tmass,weight(maxat),
     &     xref(3*maxat),cm(3),len,getlen,rmse,
     &     ub,lb,cmr(3),xav(maxat*3),x2av(maxat*3),flc(maxat),pi,
     &     gvr(3,3),xx(maxat*3),box(9),time,prec,getta,imp(maxat)

      character*80 sdum,disnam,restnam,pdbnam,rmsnam,strnam,
     &             xtcnam,tranam,flcnam

      character*100 pdboutnam
C----
C---- initialisations
C----
      iseed=741265
      nstruc=0
      prec=1000.0
      do i=1,9
         box(i)=0.0
      enddo
      box(1)=10.0
      box(5)=10.0
      box(9)=10.0
      do i=1,maxat
         imp(i)=0.0
         flc(i)=0.0
      enddo
      do i=1,maxat*3
         xav(i)=0.0
         x2av(i)=0.0
      enddo
      xtcflg=.false.
      pdbflg=.false.
      restflg=.false.
      havextc=.false.
      traflg=.false.
      verbflg=.false.
      fitflg=.false.
      notall=.false.
      disnam='dist.dat'
      strnam='dist.pdb'
      flcnam='disco.flc'
      nfrm=500
      nrit=500
      chiropt=2
      nsmoo=0
      rmsnam='disco.rms'
      nfit=0
      pi=4.0*atan(1.0)
      maxviol=0
C----
C---- check if xtc is available
C----
#     if defined (XTC)
         havextc=.true.
#     endif
C----
C---- parse command line
C----
      nargs=iargc()
      if (nargs.eq.0) call usage()
      j=0
 5    continue
      j=j+1
      call getarg(j,sdum)
      call gvstcp(sdum)
      if (sdum(1:1).ne.'-') then
         call usage()
      else if (sdum(1:2).eq.'-d') then
         if (j.ge.nargs) call usage()
         j=j+1
         call getarg(j,disnam)
      else if (sdum(1:2).eq.'-p') then
         if (j.ge.nargs) call usage()
         j=j+1
         call getarg(j,strnam)
      else if (sdum(1:2).eq.'-n') then
         if (j.ge.nargs) call usage()
         j=j+1
         call getarg(j,sdum)
         read (sdum,*) nfrm
      else if (sdum(1:2).eq.'-i') then
         if (j.ge.nargs) call usage()
         j=j+1
         call getarg(j,sdum)
         read (sdum,*) nrit
      else if (sdum(1:2).eq.'-c') then
         if (j.ge.nargs) call usage()
         j=j+1
         call getarg(j,sdum)
         read (sdum,*) chiropt
      else if (sdum(1:2).eq.'-s') then
         if (j.ge.nargs) call usage()
         j=j+1
         call getarg(j,sdum)
         read (sdum,*) iseed
      else if (sdum(1:2).eq.'-f') then
         fitflg=.true.
      else if (sdum(1:3).eq.'-bs') then
         if (j.ge.nargs) call usage()
         j=j+1
         call getarg(j,sdum)
         read (sdum,*) nsmoo
      else if (sdum(1:5).eq.'-viol') then      
         if (j.ge.nargs) call usage()
         j=j+1
         call getarg(j,sdum)
         read (sdum,*) maxviol
      else if (sdum(1:3).eq.'-op') then
         pdbflg=.true.
         if (j.ge.nargs) call usage()
         j=j+1
         call getarg(j,pdbnam)
      else if (sdum(1:3).eq.'-ox') then
         xtcflg=.true.
         if (j.ge.nargs) call usage()
         j=j+1
         call getarg(j,xtcnam)
      else if (sdum(1:3).eq.'-ot') then
         traflg=.true.
         if (j.ge.nargs) call usage()
         j=j+1
         call getarg(j,tranam)
      else if (sdum(1:3).eq.'-or') then
         if (j.ge.nargs) call usage()
         j=j+1
         call getarg(j,rmsnam)
      else if (sdum(1:3).eq.'-of') then
         if (j.ge.nargs) call usage()
         j=j+1
         call getarg(j,flcnam)
      else if (sdum(1:2).eq.'-r') then
         restflg=.true.
         if (j.ge.nargs) call usage()
         j=j+1
         call getarg(j,restnam)
      else if (sdum(1:2).eq.'-v') then
         verbflg=.true.
      else if (sdum(1:2).eq.'-h') then
         call manual()
      else
         call gvstcp(sdum)
         write(0,5001) sdum(1:40)
         write(0,*)
         call usage()
      endif

      if (j.lt.nargs) goto 5

      if ((.not.pdbflg.and..not.xtcflg).and..not.traflg) then
         write(0,5000)
         call usage()
      endif

      if (xtcflg.and..not.havextc) then
         write(0,5020)
         write(0,5021)
         stop
      endif
      if (chiropt.lt.0.or.chiropt.gt.2) then
         write(0,5030)
         write(0,5031)
         stop
      endif
C----
C---- write input params to stdout
C----
      call gvsshl(strnam)
      call gvsshl(disnam)
      call gvsshl(xtcnam)
      call gvsshl(rmsnam)
      call gvsshl(flcnam)
      call gvsshl(pdbnam)
      call gvsshl(restnam)
      call gvsshl(tranam)
      write(6,4999)
      write(6,5002) strnam(1:40)
      write(6,5004) disnam(1:40)
      if (restflg) write(0,5005) restnam(1:40)
      write(6,5006) nfrm
      write(6,5007) nrit
      write(6,5008) maxviol
      if (xtcflg) write(6,5010) xtcnam(1:40)
      if (pdbflg) write(6,5011) pdbnam(1:40)
      if (traflg) write(6,5013) tranam(1:40)
      write(6,5012) rmsnam(1:40)
      write(6,5014) flcnam(1:40)
      if (chiropt.eq.0) write(6,5017)
      if (chiropt.eq.1) write(6,5016)
      if (chiropt.eq.2) write(6,5015)
      write(6,5018) nsmoo
C----
C---- open input files
C----
      write(6,2000)
      call bbopenr(2,strnam)
      if (restflg) call bbopenr(4,restnam)
C----
C---- open output files
C----
#     if defined (XTC)
         if (xtcflg) then
            if (gvfisf(xtcnam)) call backitup(xtcnam)
            call xdrfopen(id1,xtcnam,'W',ret)
         endif
#     endif
      call bbopenw (10,rmsnam)
      call bbopenw (13,flcnam)
      if (traflg) then
         call bbopenw (11,tranam)
         write(11,*) 'trajectory generated by disco. Use XTC next time!'
      endif
C----
C---- read reference positions
C----
      nat=0
      nres=1
      cdum2='xxxx'
 15   continue
      read(2,100,err=20,end=21) sdum
      if ((sdum(1:4).ne.'ATOM').and.(sdum(1:4).ne.'HETA')) goto 15
      nat=nat+1
      if (nat.gt.maxat) then
         write(0,*) 'Increase parameter MAXAT and compile again'
         stop
      endif
      read(sdum,6000) atnam(nat),resnam(nat),chainid(nat),cdum1,
     &                xref(3*nat-2),xref(3*nat-1),xref(3*nat),
     &                weight(nat)
      if (nat.gt.1) then
         if (cdum1.ne.cdum2) nres=nres+1
      endif
      call gvsshr(atnam(nat),5)
      if (atnam(nat).eq.'   CA'.and.abs(weight(nat)).gt.0.01) then
         nfit=nfit+1
         fitsel(nat)=.true.
      else if (atnam(nat).eq.'   CA') then
         notall=.true.
      endif
      resnr(nat)=nres
      cdum2=cdum1
      goto 15
 20   write(0,*) 'error while reading:'
      write(0,*) strnam
      stop
 21   continue
      if (nat.eq.0) then
         write(0,*) 'unexpected end of file while reading:'
         write(0,*) strnam
         stop
      endif

      close(2)
      write (6,2010) nat
      write (6,2015) nres

      do i=1,nat*3
         xref(i)=xref(i)/10.0
      enddo
      if (nfit.lt.10) then
         do i=1,nat
            fitsel(i)=.false.
            if (atnam(i).eq.'   CA') fitsel(i)=.true.
         enddo
      endif
C----
C---- define improper dihedrals that are not automatically correct
C---- when all distances are correct
C----
      call defimp(nat,resnr,resnam,atnam,iimp,nimp)
      write (6,2210) nimp
      do i=1,nimp
         imp(i)=getta(iimp(1,i),iimp(2,i),iimp(3,i),iimp(4,i),xref)
         if (imp(i).lt.0.0.or.imp(i).gt.70.0) then
            write(0,5050)
            write(0,*) i,iimp(1,i),iimp(2,i),iimp(3,i),iimp(4,i),imp(i)
            stop
         endif
      enddo
C----
C---- translate center of mass
C----
      tmass=0.0
      do i=1,nat
         mass(i)=0.0
         if (abs(weight(i)).gt.0.01) then
            tmass=tmass+1.0
            mass(i)=1.0
         endif
      enddo
      if (tmass.lt.10.0) then
         tmass=float(nat)
         do i=1,nat
            mass(i)=1.0
         enddo
      endif
      call debmss (nat,xref,mass,tmass,cmr)
      do i=1,nat
         xref(3*i-2)=xref(3*i-2)-cmr(1)
         xref(3*i-1)=xref(3*i-1)-cmr(2)
         xref(3*i)=xref(3*i)-cmr(3)
      enddo
C----
C---- determine distance bounds
C----
      call getdis(disnam,in,meansig,lbub,count,xref,weight)
      write(6,2070) count
C----
C---- make tags
C----
      idum=0
      do i=1,count
         if(idum.ne.in(1,i)) tag(in(1,i))=i
         idum=in(1,i)
      enddo
      tag(nat+1)=count
      do i=1,nat
         if (tag(i).eq.0) then
            j=i
 40         continue
            j=min(j+1,nat+1)
            tag(i)=tag(j)
            if (tag(i).eq.0) goto 40
         endif
      enddo
      do i=1,nat
         if (tag(i).eq.0.and.weight(i).gt. 0.01) then
            write(0,*) 'Oops. A serious error occured.'
            write(0,*) 'Could not tag atom',i
            write(0,*) 'Please contact Bert de Groot (bgroot@gwdg.de)'
            stop
         endif
      enddo
C----
C---- bound smoothing (triangulation level)
C----
      do i=1,nsmoo
         write(6,5100)
         call smooth(count,in,lbub,tag)
      enddo
C----
C---- take square lb's and ub's
C----
      do i=1,count
         lbub(1,i)=lbub(1,i)**2
         lbub(2,i)=lbub(2,i)**2
      enddo
C----
C---- read additional restraints when required
C----
      if (restflg) then
         do i=1,100000
            read(4,*,err=45,end=46) n1,n2,lb,ub
            if (.not.had2(n1,n2,in,,count)) then
               ub=ub/10.0
               lb=lb/10.0
               count=count+1
               len=getlen(n1,n2,xref)
               in(1,count)=n1
               in(2,count)=n2
               lbub(1,count)=lb**2
               lbub(2,count)=ub**2
               meansig(1,count)=len
               meansig(2,count)=(ub-lb)/6.0
            endif
         enddo
      endif

      goto 47

 45   write(0,*) 'error while reading ',restnam(1:40)
      stop
 46   continue
      close(4)
 47   continue

      ndis=count
C----
C---- check some things
C----
#     if defined (DEBUG)
         do i=1,ndis
            n1=in(1,i)
            n2=in(1,i)
            len=getlen(n1,n2,xref)**2
            if (sqrt(lbub(1,i)).lt.0.05.or.sqrt(lbub(1,i)).gt.50.0) then
               write(0,*) 'STRANGE LB:',n1,n2,sqrt(lbub(1,i))
c               stop
            endif
            if (sqrt(lbub(2,i)).lt.0.05.or.sqrt(lbub(2,i)).gt.50.0) then
               write(0,*) 'STRANGE UB:',n1,n2,sqrt(lbub(2,i))
c               stop
            endif
            if (meansig(1,i).lt.0.05.or.meansig(1,i).gt.50.0) then
               write(0,*) 'STRANGE MEAN:',n1,n2,meansig(1,i)
c               stop
            endif
            if (meansig(2,i).lt.0.0002.or.meansig(2,i).gt.2.) then
               write(0,*) 'STRANGE SIGMA:',n1,n2,meansig(2,i)
c               stop
            endif
         enddo
#     endif

c      write(6,2071) count
      ntot=nat*(nat-1)/2
      write(6,2072) 100.0*dble(count)/dble(ntot)
C----
C---- loop over nr of output frames
C----
      do k=1,nfrm
C-------
C------- start with random coordinates!!!
C-------
 50      continue
         do i=1,nat
            if (abs(weight(i)).le.0.01) then
               do j=2,0,-1
                  x(3*i-j)=xref(3*i-j)
               enddo
            else
               do j=2,0,-1
                  x(3*i-j)=2.0*davran(iseed)-1.0
               enddo
            endif
         enddo
C-------
C------- perform correction of structure in terms of distances
C-------
         call correct(x,nat,ndis,in,lbub,meansig,nrit,nit,nviol,iseed,
     &        weight,nimp,iimp,resnr,resnam,atnam,chiropt,tag,verbflg)
C-------
C------- put structure in origin
C-------
         call debmss (nat,x,mass,tmass,cm)
         do i=1,nat
            xx(3*i-2)=x(3*i-2)-cm(1)
            xx(3*i-1)=x(3*i-1)-cm(2)
            xx(3*i)=x(3*i)-cm(3)
         enddo
C-------
C------- (least squares) fit structure to static reference
C-------
         call gvstup (nat,xref,xx,fitsel,gvr,rmse)
         rmse=rmse*10.0
C-------
C------- write the new structure
C-------
         if (nviol.le.maxviol.and.rmse.lt.10.0.and.rmse.gt.0.0) then
            if (fitflg) then
               do j=1,nat
                  x(3*j-2)=x(3*j-2)+cmr(1)
                  x(3*j-1)=x(3*j-1)+cmr(2)
                  x(3*j)=x(3*j)+cmr(3)
               enddo
            else if (.not.notall) then
               do j=1,nat*3
                  x(j)=xx(j)
               enddo
            endif
            nstruc=nstruc+1
            time=real(nstruc)
C----------
c---------- xtc structure
C----------
#           if defined (XTC)
               if (xtcflg)
     &           call writextc(id1,nat,nstruc,time,box,x,prec,ret)
#           endif
C----------
C---------- pdb structure
C----------
            if (pdbflg) then
               write(pdboutnam,6003) pdbnam,nstruc,'.pdb'
               call gvstcp(pdboutnam)
               call bbopenw(11,pdboutnam)
               write(11,6002) nstruc
               do j=1,nat
                  dum1=atnam(j)
                  call gvsshl(dum1)
                  if (gvflen(dum1).lt.4) then
                     write(cdum1,6005) dum1(1:3)
                  else
                     cdum1=dum1(1:4)
                  endif
                  write (11,6001) j,cdum1,resnam(j)(1:3),chainid(j),
     &               resnr(j),x(3*j-2)*10.,x(3*j-1)*10.,x(3*j)*10.,
     &               weight(j)
               enddo
               write(11,6004)
               close(11)
            endif
C----------
C---------- gromos trajectory
C----------
            if (traflg) write(11,120) (x(j),j=1,3*nat)
C----------
C---------- other output
C----------
            write(6,2020) nstruc,rmse
            write(10,160) nstruc,rmse
            call flush(6)
            call flush(10)
C----------
C---------- averages,fluctuations
C----------
            do i=1,3*nat
               xav(i)=xav(i)+x(i)
               x2av(i)=x2av(i)+x(i)**2
            enddo
         else
            write (6,2030) nit
            goto 50
         endif
      enddo

#     if defined (XTC)
         if (xtcflg) call xdrfclose(id1,ret)
#     endif
      close(10)
      if (traflg) close(11)
C----
C---- evaluate averages, fluctuations
C----
      do i=1,nat*3
         xav(i)=xav(i)/real(nstruc)
         x2av(i)=x2av(i)/real(nstruc)
      enddo
      do i=1,nat
         do j=2,0,-1
            flc(i)=flc(i)+x2av(3*i-j)-xav(3*i-j)**2
         enddo
         flc(i)=800.0*pi*pi*flc(i)/3.0
         dum1=atnam(i)
         call gvsshl(dum1)
         write(13,6006) i,resnam(i)(1:3),dum1(1:4),flc(i)
      enddo


      write(6,2040)

 100  format (a80)
 120  format (10f8.3)
 160  format (i5,1x,f10.5)
 2000 format ('processing input...')
 2010 format ('number of atoms.....................: ',i7)
 2015 format ('number of residues..................: ',i7)
 2020 format ('generated structure.................: ',i7,' with RMSD ',
     &        f5.2)
 2030 format ('structure rejected (no convergence after ',i5,' steps)')
 2040 format ('normal termination')
 2070 format ('number of read distances............: ',i7)
 2072 format ('perc. of total nr of distances......: ',f7.2)
 2210 format ('nr of improper dihedrals to check...: ',i7)
 2300 format ('distance between atom',i5,' and',i5,' (',f7.4,
     &         ') smaller than VDWSUM! (',f7.4,')')
 4999 format ('command line options:')
 5000 format ('disco: either -op, -ox ot -ot must be specified')
 5001 format ('disco: unknown command line argument: ',a40)
 5002 format ('input coordinates...................: ',a40)
 5004 format ('distance constraints................: ',a40)
 5005 format ('additional distance constraints.....: ',a40)
 5006 format ('nr of structures to be generated....: ',i7)
 5007 format ('maximum nr of iterations............: ',i7)
 5008 format ('maximum nr of violations............: ',i7)
 5010 format ('XTC trajectory will be written to...: ',a40)
 5011 format ('output PDB files....................: ',a40)
 5012 format ('output RMSD values..................: ',a40)
 5013 format ('output GROMOS trajectory............: ',a40)
 5014 format ('output atomic fluctuations..........: ',a40)
 5015 format ('chirality corrections will be performed on the fly')
 5016 format ('chirality corrections will be performed afterwards')
 5017 format ('no chirality corrections will be performed')
 5018 format ('Nr. of rounds of bound smoothing....: ',i7)
 5020 format ('error: -ox specified but no xtc available.') 
 5021 format ('Try -op or -ot or recompile with -DXTC')
 5030 format ('error: chirality option (-c) out of range:',i5)
 5031 format ('(should have been 0, 1 or 2)')
 5050 format ('incorrect chirality in starting structure:')
 5100 format ('bound smoothing...')
 6000 format (12x,a4,1x,a4,a1,a4,4x,3f8.3,f6.2)
 6001 format ('ATOM  ',i5,1x,a4,1x,a3,1x,a1,i4,4x,3f8.3,f6.2,'  0.00')
 6002 format ('HEADER    Structure ',i6,' generated by disco')
 6003 format (a80,i6,a4)
 6004 format ('END')
 6005 format (' ',a3)
 6006 format (i5,1x,a3,1x,a4,f10.3)

      end

      subroutine correct(x,nat,ndis,in,lbub,meansig,maxnit,nit,nviol,
     &     is,weight,nimp,iimp,resnr,resnam,atnam,chiropt,tag,verbflg)
C----
C---- makes distances correct
C----
C---- x is delivered with coordinates with correct distances
C----
C---- uses quick and dirty random number generator from numerical
C---- recipes which only works well on 32  bits operating systems
C----

      implicit none

      integer i,j,k,l,m,nat,maxnit,nit,ndis,nchk,is,flone,mask,idum,
     &     idum2,i1,i2,ifac,iran,nimp,iimp(4,*),resnr(*),chiropt,
     &     minfifteen,nviol,tag(*),low,oldlow

      include 'params.f'

      integer in(2,maxdis),ip(maxdis)

      logical viol(maxat),verbflg

      real*4  x(3*maxat),len,xx(3,maxat),half,lbub(2,maxdis),
     &     guess,dx,dy,dz,meansig(2,maxdis),
     &     gauss,rdum,fac,weight(maxat),imp(maxat)

      character*5 atnam(*),resnam(*)

      parameter(minfifteen=-15.,half=1./2.)

#     if defined (DEBUG)
         real*4 getlen
#     endif

c#     if defined (NOT32)
         real*4 davran
c#     endif

      equivalence(idum,rdum)
      parameter (i1=1664525,i2=1013904223)
      data flone /z'3F800000'/, mask /z'007FFFFF'/

C----
C---- initialize
C----
      nit=0
      nchk=ndis
      do i=1,ndis
         ip(i)=i
      enddo
      low=ndis
      oldlow=ndis
C----
C---- xx is copy of x
C----
      do i=1,nat
         do j=2,0,-1
            xx(3-j,i)=x(3*i-j)
         enddo
      enddo
C----
C---- start of iteration
C----
 10   continue
      do i=1,nat
         viol(i)=.false.
      enddo
      nit=nit+1
      nviol=0
C----
C---- randomise list
C----
      fac=real(nchk)
#     if defined (NOT32)
         do i=1,nchk
            iran=int(davran(is)*fac)+1
            idum2=ip(i)
            ip(i)=ip(iran)
            ip(iran)=idum2
         enddo
#     else
         ifac=1-nchk
         do i=1,nchk
            is=i1*is+i2
            idum=ior(flone,iand(mask,is))
            iran=int(rdum*fac)+ifac
            idum2=ip(i)
            ip(i)=ip(iran)
            ip(iran)=idum2
         enddo
#     endif
C----
C---- now walk through the list
C----
      do l=1,nchk
         k=ip(l)
         i=in(1,k)
         j=in(2,k)
         dx=xx(1,j)-xx(1,i)
         dy=xx(2,j)-xx(2,i)
         dz=xx(3,j)-xx(3,i)
         len=dx*dx+dy*dy+dz*dz
C-------
C------- if outside distance bound: make correction
C-------
         if (len.gt.lbub(2,k).or.len.lt.lbub(1,k)) then
            nviol=nviol+1
            viol(i)=.true.
            viol(j)=.true.
C----------
C---------- new distance is chosen from 'GAUSS' distribution
C----------
#           if defined (NOT32)
               gauss=-5.
               do m=1,10
                  gauss=gauss+davran(is)
               enddo
#           else
               gauss=minfifteen
               do m=1,10
                  is=i1*is+i2
                  idum=ior(flone,iand(mask,is))
                  gauss=gauss+rdum
               enddo
#           endif
C----
C---- this is the original
C----
            guess=gauss*meansig(2,k)+meansig(1,k)
c----
c---- this seems to be a good alternative. The 0.25 and 3.0 should
c---- be optimised and adapted to the system size
c----
c            if (meansig(2,k).lt.0.25) then
c               guess=gauss*meansig(2,k)+meansig(1,k)
c            else
c               guess=gauss*meansig(2,k)+3.0*davran(is)
c            endif

            len=half-half*guess/sqrt(len)
C----------
C---------- correction along connecting axis
C---------- atoms i and j are displaced by an equal amount
C----------
            dx=dx*len
            dy=dy*len
            dz=dz*len
            xx(1,i)=xx(1,i)+dx*weight(i)
            xx(2,i)=xx(2,i)+dy*weight(i)
            xx(3,i)=xx(3,i)+dz*weight(i)
            xx(1,j)=xx(1,j)-dx*weight(j)
            xx(2,j)=xx(2,j)-dy*weight(j)
            xx(3,j)=xx(3,j)-dz*weight(j)
         endif
      enddo
C----
C---- update list of distances to be checked in next round
C----
      nchk=0
      do i=1,nat
         if (viol(i)) then
            do j=tag(i),tag(i+1)-1
               nchk=nchk+1
               ip(nchk)=j
            enddo
         else
            do j=tag(i),tag(i+1)-1
               if (viol(in(2,j))) then
                  nchk=nchk+1
                  ip(nchk)=j
               endif
            enddo
         endif
      enddo
C----
C---- write info, when required
C----
#     if defined (DEBUG)
         write(6,300) nit,nchk,nviol
#     else
         if (verbflg) write(6,300) nit,nchk,nviol
#     endif
C----
C---- convergence check
C----
      low=min(low,nviol)
      if (nit.gt.10.and.nviol.gt.ndis/5) goto 40
      if (nit.gt.100.and.nviol.gt.ndis/10) goto 40
      if (mod(nit,50).eq.0) then
         if (low.gt.9*oldlow/10.and.low.gt.100) goto 40
         oldlow=low
      endif
C----
C---- if there are still violations, do another round
C----
      if (nit.lt.maxnit.and.nchk.gt.0.) then
         if (.not.(mod(nit,10).eq.0.and.chiropt.eq.2)) goto 10
      endif
C----
C---- check impropers to get chirality right
C----
      call getimpxx(nimp,iimp,xx,imp)
      j=0
      do i=1,nimp
         if (imp(i).lt.0.0.or.imp(i).gt.70.0) j=j+1
      enddo
C----
C---- if many impropers are incorrect : try mirror image
C----
      if (j.gt.nimp/2) then
         do i=1,nat
            xx(1,i)=-xx(1,i)
         enddo
         call getimpxx(nimp,iimp,xx,imp)
      endif
C----
C---- now make the corrections, if necessary
C----
      if (chiropt.eq.0) goto 20
      do i=1,nimp
         if (imp(i).lt.0.0.or.imp(i).gt.70.0) 
     &        call swapxx(iimp(3,i),resnr,resnam,atnam,xx,nat,viol)
      enddo
      if (chiropt.eq.1) goto 20
C---- 
C---- update list of distances to be checked and do another round if
C---- required
C----
      nchk=0
      do i=1,nat
         if (viol(i)) then
            do j=tag(i),tag(i+1)-1
               nchk=nchk+1
               ip(nchk)=j
            enddo
         else
            do j=tag(i),tag(i+1)-1
               if (viol(in(2,j))) then
                  nchk=nchk+1
                  ip(nchk)=j
               endif
            enddo
         endif
      enddo
      if (nit.lt.maxnit.and.nchk.gt.0) goto 10
 20   continue
C----
C---- copy xx back into x
C----
      do i=1,nat
         do j=2,0,-1
            x(3*i-j)=xx(3-j,i)
         enddo
      enddo
C----
C---- the end
C----
 40   continue
      call flush(6)
C----
C---- double check distances and impropers
C----
#     if defined (DEBUG)
         if (nchk.eq.0) then
            do k=1,ndis
               i=in(1,k)
               j=in(2,k)
               len=getlen(i,j,x)
               if (len.gt.sqrt(lbub(2,k)).or.len.lt.sqrt(lbub(1,k))) 
     &              then
                  write(0,*) 'DISTANCE OUTSIDE LIMITS',i,j,len
                  stop
               endif
            enddo
            call getimpxx(nimp,iimp,xx,imp)
            do i=1,nimp
               if (imp(i).lt.0.0.or.imp(i).gt.70.0) then
                  write(0,*) 'improper out of range:',i,imp(i)
                  stop
               endif
            enddo
         endif
#     endif

 300  format ('step',i5,', ncheck ',i8,', nviol ',i8)
 400  format ('double checking distances...')

      return

      end

      subroutine usage()

      implicit none

      write(0,100)
      write(0,110)
      write(0,120)
      write(0,125)
      write(0,*)
      write(0,130)

      stop

 100  format('Usage: disco [-d <file>] [-p <file>] [-n value] ',
     &       '[-i value] ','[-r <file>]')
 110  format('             -op <file> (or -ox <file> or -ot <file>) ',
     &     '[-or <file>]')
 120  format('             [-of <file>] [-c <value>] [-bs <value>] ',
     &     '[-s <value>]')
 125  format('             [-viol <value>] [-f] [-v] [-h]')
 130  format('disco -h will give some additional information')

      end

      subroutine manual()

      implicit none

      write(0,100)
      write(0,110)
      write(0,120)
      write(0,125)
      write(0,*)
      write(0,130)
      write(0,131)
      write(0,132)
      write(0,133)
      write(0,*)
      write(0,140)
      write(0,141)
      write(0,145)
      write(0,146)
      write(0,147)
      write(0,148)
      write(0,149)
      write(0,150)
      write(0,151)
      write(0,152)
      write(0,153)
      write(0,154)
      write(0,155)
      write(0,156)
      write(0,157)
      write(0,158)
      write(0,159)
      write(0,160)
      write(0,*)
      write(0,161)
      write(0,162)
      write(0,163)
      write(0,164)
      write(0,165)
      write(0,166)
      write(0,167)
      write(0,168)
      write(0,169)
      write(0,170)
      write(0,171)
      write(0,172)

      stop

 100  format('Usage: disco [-d <file>] [-p <file>] [-n value] ',
     &       '[-i value] ','[-r <file>]')
 110  format('             -op <file> (or -ox <file> or -ot <file>) ',
     &     '[-or <file>]')
 120  format('             [-of <file>] [-c <value>] [-bs <value>] ',
     &     '[-s <value>]')
 125  format('             [-viol <value>] [-f] [-v] [-h]')
 130  format('disco generates a number of protein conformations based')
 131  format('on a set of interatomic distance constraints (usually')
 132  format('generated by the program dist, but also additional')
 133  format('restraints may be specified).')
 140  format('Command line options:')
 141  format('Required (but defaults present, except for -op, -ot ',
     &       'and -ox):')
 145  format('-d <file>    - file containing input distances. This ',
     &       'file')
 146  format('               is usually the output of the program ',
     &       'dist. Default: dist.dat')
 147  format('-p <file>    - file containing input structure in')
 148  format('               PDB file format. This file is usually the')
 149  format('               output of the program dist.',
     &       ' Default: dist.pdb')
 150  format('-n value     - nr. of structures to be generated. ',
     &       'Default: 500')
 151  format('-i value     - maximum nr. of iterations per structure. ',
     &       'Default: 500')
 152  format('-op <file>   - basic filename for output PDB structures.')
 153  format('               (a number plus extension .pdb will be ',
     &       'added)')
 154  format('-ox <file>   - file for output trajectory in xtc format')
 155  format('-ot <file>   - file for output trajectory in formatted ',
     &       'GROMOS87 format')
 156  format('               (-op, -op or -ot must be specified)')
 157  format('-or <file>   - file with RMSD values of output ',
     &       'structures to initial ')
 158  format('               structure. Default : disco.rms')
 159  format('-of <file>   - file with mean square atomic fluctuations')
 160  format('               expressed as B-factors (default ',
     &       'disco.flc)')
 161  format('Optional:')
 162  format('-r <file>    - file containing additional distance ')
 163  format('               restraints. Default: none')
 164  format('-c <value>   - chirality check. 0 : no chirality checks;',
     &       ' 1: only check ')
 165  format('               afterwards 2: check on the fly ',
     &       '(Default :2)')
 166  format('-bs <value>  - number of rounds of bound smoothing ',
     &       '(default 0)')
 167  format('-s <value>   - initial random seed (default 741265)')
 168  format('-viol <value>- nr of acceptable violations (default 0)')
 169  format('-f           - do not fit generated structures to ',
     &       'reference')
 170  format('               (default : do fit)')
 171  format('-v           - be verbose')
 172  format('-h           - displays this message')

      end

